#include "fastlz.h"
#include <cstdlib>
#include <nds.h>
#include <filesystem.h>
#include <nds/arm9/console.h>
#include <nds/arm9/video.h>
#include <nds/debug.h>
#include <nds/ndstypes.h>
#include <stdio.h>
#include <cstdarg>

void wait_forever(const char* msg, ...);
size_t LoadFile(const char* file, unsigned char **buffer);

int main(void) {

	videoSetMode(MODE_5_3D);
	videoSetModeSub(MODE_0_2D);

	vramSetBankA(VRAM_A_MAIN_BG);
	vramSetBankB(VRAM_B_TEXTURE);
	vramSetBankF(VRAM_F_TEX_PALETTE);

	consoleDemoInit();
	consoleDebugInit(DebugDevice_NOCASH);

	nitroFSInit(NULL);

	unsigned char* bmp = NULL;
	unsigned char* bmp_decompress = (unsigned char*)malloc(256*192);
	size_t bmp_len = LoadFile("nitro:/image.bin", &bmp);

	// size_t bmp_decompress_len = bmp[0] | bmp[1] << 8;
	size_t bmp_decompress_len = 0;

	memcpy(&bmp_decompress_len, bmp, sizeof(uint32));
	fprintf(stderr, "%i %i\n", bmp_decompress_len, bmp_len);
	fastlz_decompress(&bmp[4], bmp_decompress_len, bmp_decompress, 256*192);

	unsigned char* pal = NULL;
	size_t pal_len = LoadFile("nitro:/palette.bin", &pal);

	int bgp = bgInit(3, BgType_Bmp8, BgSize_B8_256x256, 0, 0);

	// DC_FlushRange(bg, bg_len);
	dmaCopy(bmp_decompress, bgGetGfxPtr(bgp), 256*192);
	dmaCopy(pal, BG_PALETTE, pal_len);
	nocashMessage("done\n");

	while(1) {

		swiWaitForVBlank();
		scanKeys();
	}

	return 0;
}

size_t LoadFile(const char* file, unsigned char **buffer)
{
	FILE *in = fopen(file, "rb");
	if (in == NULL) wait_forever("cannot load %s", file);

	fseek(in, 0, SEEK_END);
	long file_size = ftell(in);
	rewind(in);

	*buffer = (unsigned char*)malloc(file_size);
	if (*buffer == NULL) wait_forever("Failed to malloc %s", file);

	size_t bytes_read = fread(*buffer, 1, file_size, in);
	if (bytes_read != file_size)
	{
		fclose(in);
		free(*buffer);
		wait_forever("Failed to read %s", file);
	}

	return bytes_read;
}

void wait_forever(const char* msg, ...)
{
	va_list args;
	va_start(args, msg);
	fprintf(stdout, msg, args);
	fprintf(stderr, msg, args);
	va_end(args);
  while (1) swiWaitForVBlank();
}